use std::sync::{Arc, Mutex};
use actix_web::{App, HttpServer, web};
use actix_web::web::Data;
use backend::endpoints::benchmark::benchmark;
use backend::endpoints::{auth::*};
use backend::storages::postgres::administrator::PostgresAdministratorRepository;
use dotenv_codegen::dotenv;
use backend::{State, services};
use mongodb::Client;
use mongodb::options::ClientOptions;
use tokio_postgres::NoTls;
use backend::storages::postgres::car::PostgresCarRepository;
use backend::storages::postgres::car_station::PostgresCarStationRepository;
use backend::storages::postgres::client::PostgresClientRepository;
use backend::storages::postgres::rent::PostgresRentRepository;
use backend::storages::postgres::owner::PostgresOwnerRepository;
use backend::storages::postgres::benchmark::PostgresBenchmarkRepository;
use backend::storages::mongo::car::MongoCarRepository;
use backend::storages::mongo::car_station::MongoCarStationRepository;
use backend::storages::mongo::administrator::MongoAdministratorRepository;
use backend::storages::mongo::client::MongoClientRepository;
use backend::storages::mongo::rent::MongoRentRepository;
use backend::storages::mongo::owner::MongoOwnerRepository;
use backend::storages::mongo::benchmark::MongoBenchmarkRepository;
use backend::storages::*;
use backend::endpoints::car_station::*;
use backend::endpoints::car::*;
use backend::endpoints::client::*;
use backend::endpoints::rent::*;
use backend::endpoints::owner::*;

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    // let (client, connection) = tokio_postgres::connect(
    //     &format!("host={} user={} password={} dbname={}", dotenv!("HOST"), dotenv!("USERR"), dotenv!("PASSWORD"), dotenv!("DBNAMEE")),
    //     NoTls
    // ).await.unwrap();

    // tokio::spawn(async move {
    //     if let Err(e) = connection.await {
    //         eprintln!("connection error: {}", e);
    //     }
    // });

    // let client = Arc::new(client);

    // let car_repository = PostgresCarRepository { connection: Arc::clone(&client) };

    // let car_station_repository = PostgresCarStationRepository { connection: Arc::clone(&client) };

    // let client_repository = PostgresClientRepository { connection: Arc::clone(&client) };

    // let owner_repository = PostgresOwnerRepository { connection: Arc::clone(&client) };

    // let rent_repository = PostgresRentRepository { connection: Arc::clone(&client) };

    // let administrator_repository = PostgresAdministratorRepository { connection: Arc::clone(&client) };

    // let benchmark_repository = PostgresBenchmarkRepository { connection: Arc::clone(&client) };

    let client_options = ClientOptions::parse(dotenv!("MONGO")).await.unwrap();

    let client = Client::with_options(client_options).unwrap();

    let db = Arc::new(client.database("default_db"));

    let car_repository = MongoCarRepository { database: Arc::clone(&db) };

    let car_station_repository = MongoCarStationRepository { database: Arc::clone(&db) };

    let client_repository = MongoClientRepository { database: Arc::clone(&db) };

    let owner_repository = MongoOwnerRepository { database: Arc::clone(&db) };

    let rent_repository = MongoRentRepository { database: Arc::clone(&db) };

    let administrator_repository = MongoAdministratorRepository { database: Arc::clone(&db) };

    let benchmark_repository = MongoBenchmarkRepository { database: Arc::clone(&db) };
    
    let auth_service = services::auth::AuthService::new();

    let state = Data::new(Mutex::new(State {
        car_repository: Box::new(car_repository),
        car_station_repository: Box::new(car_station_repository),
        client_repository: Box::new(client_repository),
        owner_repository: Box::new(owner_repository),
        rent_repository: Box::new(rent_repository),
        administrator_repository: Box::new(administrator_repository),
        benchmark_repository: Box::new(benchmark_repository),
        auth_service
    }));
    
    HttpServer::new(move || {
        App::new()
            .app_data(Data::clone(&state))
            .service(web::scope("/api")
                .service(
                    web::scope("/cars")
                        .service(get_cars)
                        .service(get_report)
                        .service(get_car)
                        .service(create_car)
                        .service(update_car)
                        .service(delete_car)
                )
                .service(
                    web::scope("/clients")
                        .service(get_clients)
                        .service(get_client)
                        .service(create_client)
                        .service(update_client)
                        .service(delete_client)
                )
                .service(
                    web::scope("/owners")
                        .service(get_owners)
                        .service(get_owner)
                        .service(create_owner)
                        .service(update_owner)
                        .service(delete_owner)
                )
                .service(
                    web::scope("/car_stations")
                        .service(get_car_stations)
                        .service(get_car_station)
                        .service(create_car_station)
                        .service(update_car_station)
                        .service(delete_car_station)
                )
                .service(
                    web::scope("/rents")
                        .service(get_rents)
                        .service(get_rent)
                        .service(create_rent)
                        .service(update_rent)
                        .service(delete_rent)
                        .service(end_rent)
                )
                .service(login)
                .service(logout)
            ).service(benchmark)
    })
        .bind("0.0.0.0:8080")?
        .run()
        .await
}