use mongodb::Database;
use mongodb::bson::oid::ObjectId;
use mongodb::bson::{doc, Document, Bson};
use futures::stream::{TryStreamExt, StreamExt};
use mongodb::options::FindOptions;
use std::sync::Arc;
use crate::models::car_station::*;
use crate::storages::traits::CarStationRepository;
use async_trait::async_trait;


pub struct MongoCarStationRepository {
    pub database: Arc<Database>
}

#[async_trait]
impl CarStationRepository for MongoCarStationRepository {
    async fn create(&self, car_station: BindingCarStation) -> Result<CarStation, String> {
        let collection = self.database.collection::<Document>("car_stations");
        let document = doc! {
            "address": car_station.address.clone()
        };
        let result = collection.insert_one(document, None).await;
        match result {
            Ok(result) => {
                let id = result.inserted_id.as_object_id().unwrap().to_hex();
                let car_station = CarStation { id, address: car_station.address };
                Ok(car_station)
            },
            Err(e) => Err(e.to_string())
        }
    }

    async fn read(&self, id: String) -> Result<CarStation, String> {
        let collection = self.database.collection::<Document>("car_stations");
        let filter = doc! {
            "_id": ObjectId::parse_str(&id).unwrap()
        };
        let result = collection.find_one(filter, None).await;
        match result {
            Ok(Some(document)) => {
                let id = document.get_object_id("_id").unwrap().to_hex();
                let address = document.get_str("address").unwrap().to_string();
                let car_station = CarStation { id, address };
                Ok(car_station)
            },
            Ok(None) => Err("Car station not found".to_string()),
            Err(e) => Err(e.to_string())
        }
    }

    async fn read_all(&self) -> Result<Vec<CarStation>, String> {
        let collection = self.database.collection::<Document>("car_stations");
        let mut cursor = collection.find(None, None).await.map_err(|e| e.to_string())?;
        let mut car_stations = Vec::new();
        while let Some(result) = cursor.next().await {
            match result {
                Ok(document) => {
                    let id = document.get_object_id("_id").unwrap().to_hex();
                    let address = document.get_str("address").unwrap().to_string();
                    let car_station = CarStation { id, address };
                    car_stations.push(car_station);
                },
                Err(e) => return Err(e.to_string())
            }
        }
        Ok(car_stations)
    }

    async fn update(&self, id: String, car_station: BindingCarStation) -> Result<CarStation, String> {
        let collection = self.database.collection::<Document>("car_stations");
        let filter = doc! {
            "_id": ObjectId::parse_str(&id).unwrap()
        };
        let update = doc! {
            "$set": {
                "address": car_station.address.clone()
            }
        };
        let result = collection.update_one(filter, update, None).await;
        match result {
            Ok(_) => {
                let car_station = CarStation { id, address: car_station.address };
                Ok(car_station)
            },
            Err(e) => Err(e.to_string())
        }
    }

    async fn delete(&self, id: String) -> Result<(), String> {
        let collection = self.database.collection::<Document>("car_stations");
        let filter = doc! {
            "_id": ObjectId::parse_str(&id).unwrap()
        };
        let result = collection.delete_one(filter, None).await;
        match result {
            Ok(_) => Ok(()),
            Err(e) => Err(e.to_string())
        }
    }
}