use mongodb::Database;
use mongodb::bson::oid::ObjectId;
use mongodb::bson::{doc, Document, Bson};
use futures::stream::{TryStreamExt, StreamExt};
use mongodb::options::{FindOptions, AggregateOptions};
use rust_decimal::Decimal;
use rust_decimal::prelude::{ToPrimitive, FromPrimitive};
use std::default;
use std::sync::Arc;
use crate::models::client::*;
use crate::storages::traits::OwnerRepository;
use async_trait::async_trait;



pub struct MongoOwnerRepository {
    pub database: Arc<Database>
}

#[async_trait]
impl OwnerRepository for MongoOwnerRepository {
    async fn create(&self, owner: BindingClient) -> Result<Client, String> {
        let collection = self.database.collection::<Document>("clients");
        let document = doc! {
            "name": owner.name.clone(),
            "surname": owner.surname.clone(),
            "middlename": owner.middlename.clone(),
            "phone": owner.phone.clone(),
            "cars": Bson::Array(Vec::<Bson>::new()),
            "rents": Bson::Array(Vec::<Bson>::new())
        };
        let result = collection.insert_one(document, None).await;
        match result {
            Ok(result) => {
                let id = result.inserted_id.as_object_id().unwrap().to_hex();
                let owner = Client { id, name: owner.name, surname: owner.surname, middlename: owner.middlename, phone: owner.phone };
                Ok(owner)
            },
            Err(e) => Err(e.to_string())
        }
    }

    async fn read(&self, id: String) -> Result<Client, String> {
        let collection = self.database.collection::<Document>("clients");
        let filter = doc! {
            "_id": ObjectId::parse_str(&id).unwrap()
        };
        let result = collection.find_one(filter, None).await;
        match result {
            Ok(Some(document)) => {
                let id = document.get_object_id("_id").unwrap().to_hex();
                let name = document.get_str("name").unwrap().to_string();
                let surname = document.get_str("surname").unwrap().to_string();
                let middlename = document.get_str("middlename").map(|s| s.to_string());
                let phone = document.get_str("phone").unwrap().to_string();
                let owner = Client { id, name, surname, middlename: Some(middlename.unwrap()), phone };
                Ok(owner)
            },
            Ok(None) => Err("Owner not found".to_string()),
            Err(e) => Err(e.to_string())
        }
    }

    async fn read_all(&self) -> Result<Vec<Client>, String> {
        let collection = self.database.collection::<Document>("clients");
        let mut cursor = collection.find(doc! {"cars": { "$exists": true }}, None).await.map_err(|e| e.to_string())?;
        let mut owners = Vec::new();
        while let Some(result) = cursor.next().await {
            match result {
                Ok(document) => {
                    let id = document.get_object_id("_id").unwrap().to_hex();
                    let name = document.get_str("name").unwrap().to_string();
                    let surname = document.get_str("surname").unwrap().to_string();
                    let middlename = document.get_str("middlename").map(|s| s.to_string());
                    let phone = document.get_str("phone").unwrap().to_string();
                    let owner = Client { id, name, surname, middlename: Some(middlename.unwrap()), phone };
                    owners.push(owner);
                },
                Err(e) => return Err(e.to_string())
            }
        }
        Ok(owners)
    }

    async fn update(&self, id: String, owner: BindingClient) -> Result<Client, String> {
        let collection = self.database.collection::<Document>("clients");
        let filter = doc! {
            "_id": ObjectId::parse_str(&id).unwrap()
        };
        let update = doc! {
            "$set": {
                "name": owner.name.clone(),
                "surname": owner.surname.clone(),
                "middlename": owner.middlename.clone(),
                "phone": owner.phone.clone()
            }
        };
        let result = collection.update_one(filter, update, None).await;
        match result {
            Ok(result) => {
                if result.modified_count == 1 {
                    let owner = self.read(id).await?;
                    Ok(owner)
                } else {
                    Err("Owner not found".to_string())
                }
            },
            Err(e) => Err(e.to_string())
        }
    }

    async fn delete(&self, id: String) -> Result<(), String> {
        let collection = self.database.collection::<Document>("owners");
        let filter = doc! {
            "_id": ObjectId::parse_str(&id).unwrap()
        };
        let result = collection.delete_one(filter, None).await;
        match result {
            Ok(result) => {
                if result.deleted_count == 1 {
                    Ok(())
                } else {
                    Err("Owner not found".to_string())
                }
            },
            Err(e) => Err(e.to_string())
        }
    }
}
